import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
/* eslint-disable react/no-array-index-key */
import { useMemo } from 'react';
import { AlignGroup } from 'react-d3-utils';
import { useLegend } from '../legendContext';
import { usePlotContext } from '../plotContext';
import { markersComps } from './Markers';
function exclusiveProps(margins, a, b, position) {
    if (margins[a] !== undefined) {
        if (margins[b] !== undefined) {
            throw new Error(`${a} and ${b} should't be both defined for the position ${position}`);
        }
        return { key: a, value: margins[a] };
    }
    return margins[b] !== undefined ? { key: b, value: margins[b] } : {};
}
function translation(position, legendMargins, plotWidth, plotHeight) {
    switch (position) {
        case 'embedded': {
            const { key: verticalKey = 'top', value: verticalValue = 10 } = exclusiveProps(legendMargins, 'top', 'bottom', position);
            const { key: horizontalKey = 'left', value: horizontalValue = 10 } = exclusiveProps(legendMargins, 'left', 'right', position);
            const x = horizontalKey === 'right'
                ? plotWidth - horizontalValue
                : horizontalValue;
            const y = verticalKey === 'bottom' ? plotHeight - verticalValue : verticalValue;
            return {
                x,
                y,
                horizontalAlign: horizontalKey === 'left' ? 'start' : 'end',
                verticalAlign: verticalKey === 'top' ? 'start' : 'end',
            };
        }
        case 'top': {
            const { value: horizontalValue = plotWidth / 2 } = exclusiveProps(legendMargins, 'left', 'right', position);
            const x = horizontalValue;
            const y = -legendMargins.bottom || 0;
            return { x, y, horizontalAlign: 'middle', verticalAlign: 'end' };
        }
        case 'bottom': {
            const { value: horizontalValue = plotWidth / 2 } = exclusiveProps(legendMargins, 'left', 'right', position);
            const x = horizontalValue;
            const y = plotHeight + (legendMargins.top || 0);
            return { x, y, horizontalAlign: 'middle', verticalAlign: 'start' };
        }
        case 'left': {
            const { key: verticalKey = 'top', value: verticalValue = plotHeight / 2, } = exclusiveProps(legendMargins, 'top', 'bottom', position);
            const y = verticalKey === 'bottom' ? -verticalValue : verticalValue;
            const x = -legendMargins.right || 0;
            return { x, y, horizontalAlign: 'end', verticalAlign: 'middle' };
        }
        case 'right': {
            const { key: verticalKey = 'top', value: verticalValue = plotHeight / 2, } = exclusiveProps(legendMargins, 'top', 'bottom', position);
            const y = verticalKey === 'bottom' ? -verticalValue : verticalValue;
            const x = plotWidth + (legendMargins.left || 0);
            return { x, y, horizontalAlign: 'start', verticalAlign: 'middle' };
        }
        default: {
            throw new Error(`Position ${JSON.stringify(position)} unknown`);
        }
    }
}
export function Legend({ position, ...legendMargins }) {
    const { plotWidth, plotHeight } = usePlotContext();
    const [state] = useLegend();
    const alignGroupProps = useMemo(() => {
        return translation(position, legendMargins, plotWidth, plotHeight);
    }, [position, legendMargins, plotWidth, plotHeight]);
    return (_jsx(AlignGroup, { ...alignGroupProps, children: state.labels.map((value, index) => {
            const xPos = 10;
            const yPos = (index + 1) * 16 - xPos + 5;
            if (value.range) {
                return (_jsxs("g", { transform: `translate(${xPos}, ${0})`, children: [getRangeShape({
                            index,
                            rangeColor: value.range.rangeColor,
                            lineColor: value.colorLine,
                        }), _jsx("text", { x: 30, y: `${index + 1}em`, children: value.label }, `text-${value.label}-${index}`)] }, `${value.colorLine}/${value.range.rangeColor}-${value.label}`));
            }
            const Marker = markersComps[value.shape.figure];
            return (_jsxs("g", { transform: `translate(${xPos}, ${0})`, children: [getLineShape({ index, color: value.colorLine }), _jsx("g", { transform: `translate(${xPos - 1}, ${yPos})`, children: !value.shape.hidden && (_jsx(Marker, { size: 10, style: { fill: value.shape.color } }, void 0)) }, void 0), _jsx("text", { x: 30, y: `${index + 1}em`, children: value.label }, `text-${value.label}-${index}`)] }, `${value.colorLine}/${value.shape.color}-${value.label}`));
        }) }, void 0));
}
function getLineShape(config) {
    const x = 0;
    const y = (config.index + 1) * 16 - x - 7;
    return _jsx("rect", { x: x, y: y, width: "20", height: "3", fill: config.color }, void 0);
}
function getRangeShape(config) {
    const x = 0;
    const y = (config.index + 1) * 16 - x - 12;
    return (_jsxs("g", { transform: `translate(${x}, ${y})`, children: [_jsx("rect", { width: "20", height: "10", fill: config.rangeColor }, void 0), _jsx("line", { x1: 0, y: 0, x2: 20, y2: 0, stroke: config.lineColor, strokeWidth: 3 }, void 0), _jsx("line", { x1: 0, y1: 10, x2: 20, y2: 10, stroke: config.lineColor, strokeWidth: 3 }, void 0)] }, void 0));
}
//# sourceMappingURL=Legend.js.map