import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { scaleOrdinal } from 'd3-scale';
import { schemeSet1 } from 'd3-scale-chromatic';
import { produce } from 'immer';
import { useMemo, useReducer } from 'react';
import { useBBoxObserver } from 'react-d3-utils';
import { bboxContext } from '../bboxContext';
import { LegendProvider } from '../legendContext';
import { plotContext, plotDispatchContext, plotReducer, useAxisContext, } from '../plotContext';
import { splitChildren } from '../utils/splitChildren';
import Tracking from './Tracking';
import TransparentRect from './TransparentRect';
const reducerCurr = produce(plotReducer);
const initialState = {
    series: [],
    axis: {},
};
const defaultSvgStyle = {
    overflow: 'visible',
    fontFamily: 'Arial, Helvetica, sans-serif',
};
export function Plot(props) {
    const { width, height, colorScheme = schemeSet1, margin = {}, svgStyle = {}, svgId, svgClassName, plotViewportStyle = {}, seriesViewportStyle = {}, onMouseMove, onClick, onMouseEnter, onMouseLeave, onMouseDown, onMouseUp, onDoubleClick, children, } = props;
    const [state, dispatch] = useReducer(reducerCurr, initialState, undefined);
    const { seriesAndAnnotations, topAxis, rightAxis, bottomAxis, leftAxis, topHeading, bottomHeading, legend, } = splitChildren(children);
    if (width === undefined) {
        throw new Error('width is mandatory');
    }
    if (height === undefined) {
        throw new Error('height is mandatory');
    }
    // Bounding boxes used to adapt viewport size.
    const headingBbox = useBBoxObserver();
    const topAxisBbox = useBBoxObserver();
    const rightAxisBbox = useBBoxObserver();
    const bottomAxisBbox = useBBoxObserver();
    const leftAxisBbox = useBBoxObserver();
    // Distances in plot
    const { left = 0, right = 0, top = 0, bottom = 0 } = margin;
    const plotWidth = width - left - leftAxisBbox.width - right - rightAxisBbox.width;
    const plotHeight = height -
        top -
        headingBbox.height -
        topAxisBbox.height -
        bottom -
        bottomAxisBbox.height;
    const leftOffset = left + leftAxisBbox.width;
    const topOffset = top + (topHeading ? headingBbox.height : 0) + topAxisBbox.height;
    // Set scales
    const axisContext = useAxisContext(state, { plotWidth, plotHeight });
    const labels = useMemo(() => state.series.map(({ id, label }) => ({ id, label })), [state.series]);
    const ids = useMemo(() => state.series.map(({ id }) => id), [state.series]);
    const colorScaler = useMemo(() => {
        return scaleOrdinal().range(colorScheme).domain(ids);
    }, [colorScheme, ids]);
    return (_jsx(plotContext.Provider, { value: {
            width,
            height,
            left,
            right,
            top,
            bottom,
            plotWidth,
            plotHeight,
            labels,
            colorScaler,
            axisContext,
        }, children: _jsx(plotDispatchContext.Provider, { value: dispatch, children: _jsx(LegendProvider, { children: _jsxs("svg", { xmlns: "http://www.w3.org/2000/svg", width: width, height: height, style: { ...defaultSvgStyle, ...svgStyle }, id: svgId, className: svgClassName, children: [_jsx(TransparentRect, { width: width, height: height, style: plotViewportStyle }, void 0), _jsxs("g", { transform: `translate(${leftOffset}, ${topOffset})`, children: [_jsx(TransparentRect, { width: plotWidth, height: plotHeight, style: seriesViewportStyle }, void 0), _jsx("clipPath", { id: "seriesViewportClip", children: _jsx("rect", { width: plotWidth, height: plotHeight }, void 0) }, void 0), _jsx("g", { style: { clipPath: 'url(#seriesViewportClip)' }, children: seriesAndAnnotations }, void 0), _jsx(bboxContext.Provider, { value: topAxisBbox.ref, children: _jsx("g", { children: topAxis }, void 0) }, void 0), _jsx(bboxContext.Provider, { value: rightAxisBbox.ref, children: _jsx("g", { children: rightAxis }, void 0) }, void 0), _jsx(bboxContext.Provider, { value: bottomAxisBbox.ref, children: _jsx("g", { children: bottomAxis }, void 0) }, void 0), _jsx(bboxContext.Provider, { value: leftAxisBbox.ref, children: _jsx("g", { children: leftAxis }, void 0) }, void 0), legend, onClick || onMouseMove ? (_jsx(Tracking, { stateSeries: state.series, onClick: (position) => onClick === null || onClick === void 0 ? void 0 : onClick(position), onMouseMove: (position) => onMouseMove === null || onMouseMove === void 0 ? void 0 : onMouseMove(position), onMouseEnter: onMouseEnter, onMouseLeave: onMouseLeave, onMouseDown: onMouseDown, onMouseUp: onMouseUp, onDoubleClick: onDoubleClick }, void 0)) : null] }, void 0), _jsx("g", { ref: headingBbox.ref, children: topHeading || bottomHeading }, void 0)] }, void 0) }, void 0) }, void 0) }, void 0));
}
//# sourceMappingURL=Plot.js.map