"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BarSeries = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const legendContext_1 = require("../legendContext");
const plotContext_1 = require("../plotContext");
const utils_1 = require("../utils");
const ScatterSeries_1 = require("./ScatterSeries");
function BarSeries(props) {
    var _a, _b;
    const [, legendDispatch] = (0, legendContext_1.useLegend)();
    const { colorScaler } = (0, plotContext_1.usePlotContext)();
    const [id] = (0, react_1.useState)(() => props.groupId || `series-${(0, utils_1.getNextId)()}`);
    const { lineStyle = {}, displayMarker = false, ...otherProps } = props;
    const lineProps = {
        id,
        data: props.data,
        xAxis: props.xAxis || 'x',
        yAxis: props.yAxis || 'y',
        lineStyle,
    };
    const colorLine = (lineStyle === null || lineStyle === void 0 ? void 0 : lineStyle.stroke)
        ? lineStyle === null || lineStyle === void 0 ? void 0 : lineStyle.stroke.toString()
        : colorScaler(id);
    const color = ((_b = (_a = otherProps.markerStyle) === null || _a === void 0 ? void 0 : _a.fill) === null || _b === void 0 ? void 0 : _b.toString()) || colorScaler(id);
    const figure = otherProps.markerShape || 'circle';
    const shape = (0, react_1.useMemo)(() => {
        return {
            color,
            figure,
            hidden: !displayMarker,
        };
    }, [color, displayMarker, figure]);
    (0, react_1.useEffect)(() => {
        legendDispatch({
            type: 'ADD_LEGEND_LABEL',
            payload: {
                id,
                label: otherProps.label,
                colorLine,
                shape,
            },
        });
        return () => legendDispatch({ type: 'REMOVE_LEGEND_LABEL', payload: { id } });
    }, [colorLine, legendDispatch, otherProps.label, shape, id]);
    return ((0, jsx_runtime_1.jsxs)("g", { children: [props.hidden ? null : (0, jsx_runtime_1.jsx)(BarSeriesRender, { ...lineProps }, void 0), (0, jsx_runtime_1.jsx)(ScatterSeries_1.ScatterSeries, { ...otherProps, hidden: !displayMarker || props.hidden, groupId: id }, void 0)] }, void 0));
}
exports.BarSeries = BarSeries;
function BarSeriesRender({ id, data, xAxis, yAxis, lineStyle, }) {
    // Get scales from context
    const { axisContext, colorScaler } = (0, plotContext_1.usePlotContext)();
    const [xScale, yScale] = (0, utils_1.validateAxis)(axisContext, xAxis, yAxis);
    const color = colorScaler(id);
    if (xScale === undefined || yScale === undefined) {
        return null;
    }
    // default style
    const style = {
        stroke: color,
        strokeWidth: 2,
        ...lineStyle,
    };
    return ((0, jsx_runtime_1.jsx)("g", { children: data.map(({ x, y }) => ((0, jsx_runtime_1.jsx)("line", { style: style, x1: xScale(x), x2: xScale(x), y1: yScale(y), y2: yScale(0) }, `${x}-${y}`))) }, void 0));
}
