"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const plotContext_1 = require("../plotContext");
const utils_1 = require("../utils");
function ErrorBars(props) {
    const { xAxis = 'x', yAxis = 'y', data, hidden, ...otherProps } = props;
    const { axisContext } = (0, plotContext_1.usePlotContext)();
    const [xScale, yScale] = (0, utils_1.validateAxis)(axisContext, xAxis, yAxis);
    const points = (0, react_1.useMemo)(() => {
        if (hidden)
            return null;
        if (xScale === undefined || yScale === undefined) {
            return null;
        }
        const pointBars = data.map((point, i) => {
            const xError = (0, utils_1.validateSeriesPointError)(point.xError);
            const yError = (0, utils_1.validateSeriesPointError)(point.yError);
            return ((0, jsx_runtime_1.jsx)(PointBars // eslint-disable-next-line react/no-array-index-key
            , { origin: { x: xScale(point.x), y: yScale(point.y) }, bottom: yError ? yScale(point.y - yError[0]) : null, top: yError ? yScale(point.y + yError[1]) : null, left: xError ? xScale(point.x - xError[0]) : null, right: xError ? xScale(point.x + xError[1]) : null, ...otherProps }, `ErrorBars-${i}`));
        });
        return pointBars;
    }, [data, xScale, yScale, hidden, otherProps]);
    return (0, jsx_runtime_1.jsx)("g", { children: points }, void 0);
}
exports.default = ErrorBars;
function PointBars(props) {
    const { origin, top, bottom, left, right, style, capSize = 10, capStyle, } = props;
    const defaultColor = 'black';
    return ((0, jsx_runtime_1.jsxs)("g", { children: [top != null && ((0, jsx_runtime_1.jsxs)("g", { children: [(0, jsx_runtime_1.jsx)("line", { x1: origin.x, x2: origin.x, y1: origin.y, y2: top, stroke: defaultColor, ...style }, void 0), (0, jsx_runtime_1.jsx)("line", { x1: origin.x - capSize / 2, x2: origin.x + capSize / 2, y1: top, y2: top, stroke: defaultColor, ...style, ...capStyle }, void 0)] }, void 0)), bottom != null && ((0, jsx_runtime_1.jsxs)("g", { children: [(0, jsx_runtime_1.jsx)("line", { x1: origin.x, x2: origin.x, y1: origin.y, y2: bottom, stroke: defaultColor, ...style }, void 0), (0, jsx_runtime_1.jsx)("line", { x1: origin.x - capSize / 2, x2: origin.x + capSize / 2, y1: bottom, y2: bottom, stroke: defaultColor, ...style, ...capStyle }, void 0)] }, void 0)), left != null && ((0, jsx_runtime_1.jsxs)("g", { children: [(0, jsx_runtime_1.jsx)("line", { x1: origin.x, x2: left, y1: origin.y, y2: origin.y, stroke: defaultColor, ...style }, void 0), (0, jsx_runtime_1.jsx)("line", { x1: left, x2: left, y1: origin.y - capSize / 2, y2: origin.y + capSize / 2, stroke: defaultColor, ...style, ...capStyle }, void 0)] }, void 0)), right != null && ((0, jsx_runtime_1.jsxs)("g", { children: [(0, jsx_runtime_1.jsx)("line", { x1: origin.x, x2: right, y1: origin.y, y2: origin.y, stroke: defaultColor, ...style }, void 0), (0, jsx_runtime_1.jsx)("line", { x1: right, x2: right, y1: origin.y - capSize / 2, y2: origin.y + capSize / 2, stroke: defaultColor, ...style, ...capStyle }, void 0)] }, void 0))] }, void 0));
}
