"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.usePointPosition = exports.useEllipsePosition = exports.useRectanglePosition = exports.usePositionAndSize = exports.usePosition = void 0;
const plotContext_1 = require("./plotContext");
const utils_1 = require("./utils");
function usePosition(config) {
    const { axisContext } = (0, plotContext_1.usePlotContext)();
    const [xScale, yScale] = (0, utils_1.validateAxis)(axisContext, 'x', 'y');
    const { x, y } = config;
    return {
        x: convertValue(x, xScale),
        y: convertValue(y, yScale),
    };
}
exports.usePosition = usePosition;
function usePositionAndSize(config) {
    const { axisContext } = (0, plotContext_1.usePlotContext)();
    const [xScale, yScale] = (0, utils_1.validateAxis)(axisContext, 'x', 'y');
    const { x, y, width, height } = config;
    return {
        x: convertValue(x, xScale),
        y: convertValue(y, yScale),
        width: convertValueAbs(width, xScale),
        height: convertValueAbs(height, yScale),
    };
}
exports.usePositionAndSize = usePositionAndSize;
function useRectanglePosition(config) {
    const { axisContext } = (0, plotContext_1.usePlotContext)();
    const [xScale, yScale] = (0, utils_1.validateAxis)(axisContext, 'x', 'y');
    const { x1, y1, x2, y2 } = config;
    return {
        x: convertMinValue(x1, x2, xScale),
        y: convertMinValue(y1, y2, yScale),
        width: convertDimensions(x1, x2, xScale),
        height: convertDimensions(y1, y2, yScale),
    };
}
exports.useRectanglePosition = useRectanglePosition;
function useEllipsePosition(props) {
    const { axisContext } = (0, plotContext_1.usePlotContext)();
    const [xScale, yScale] = (0, utils_1.validateAxis)(axisContext, 'x', 'y');
    const { cx, cy, rx, ry } = props;
    return {
        cx: convertValue(cx, xScale),
        cy: convertValue(cy, yScale),
        rx: convertValueAbs(rx, xScale),
        ry: convertValueAbs(ry, yScale),
    };
}
exports.useEllipsePosition = useEllipsePosition;
function convertValue(value, scale) {
    if (scale === undefined)
        return 0;
    return typeof value === 'number' ? scale(value) : Number(value);
}
function convertMinValue(value1, value2, scale) {
    if (scale === undefined)
        return 0;
    return Math.min(typeof value2 === 'number' ? scale(value2) : parseInt(value2, 10), typeof value1 === 'number' ? scale(value1) : parseInt(value1, 10));
}
function convertValueAbs(value, scale) {
    if (scale === undefined)
        return 0;
    return typeof value === 'number' ? Math.abs(scale(0) - scale(value)) : value;
}
function convertDimensions(value1, value2, scale) {
    if (scale === undefined)
        return 0;
    return Math.abs((typeof value2 === 'number' ? scale(value2) : parseInt(value2, 10)) -
        (typeof value1 === 'number' ? scale(value1) : parseInt(value1, 10)));
}
function usePointPosition(config) {
    const { axisContext } = (0, plotContext_1.usePlotContext)();
    const [xScale, yScale] = (0, utils_1.validateAxis)(axisContext, 'x', 'y');
    const points = config;
    return points
        .map((point) => `${convertValue(point.x, xScale)},${convertValue(point.y, yScale)}`)
        .join(' ');
}
exports.usePointPosition = usePointPosition;
